import { render, screen, waitFor } from 'test-utils'
import AdapterDateFns from '@mui/lab/AdapterDateFns'
import { LocalizationProvider } from '@mui/lab'
import { addYears } from 'date-fns/fp'
import { FeatureContext } from 'contexts'
import GuestRegisterStep from './register'
import { GuestRegisterData } from '../enteredGuestData'
import AuthenticationMethod from '../authenticationMethod'
import { GuestInviteInformation } from '../guestDataForm'

function getEmptyGuestData(): GuestInviteInformation {
  return {
    first_name: '',
    last_name: '',
    ou_name_en: '',
    ou_name_nb: '',
    role_name_en: '',
    role_name_nb: '',
    role_start: '',
    role_end: '',
    email: '',
    mobile_phone: '',
    date_of_birth: null,
    fnr: '',
    passport: '',
    countryForCallingCode: '',
    authentication_method: AuthenticationMethod.Invite,
    gender: '',
  }
}

test('Guest register page showing passport field on manual registration', async () => {
  const nextHandler = (registerData: GuestRegisterData) => {
    console.log(`Entered data: ${registerData}`)
  }

  render(
    <LocalizationProvider dateAdapter={AdapterDateFns}>
      <GuestRegisterStep
        nextHandler={nextHandler}
        initialGuestData={getEmptyGuestData()}
        registerData={null}
      />
    </LocalizationProvider>
  )

  await waitFor(() => {
    expect(screen.queryByTestId('passport_number_input')).toBeInTheDocument()
  })
})

test('Name not editable if present and invite is Feide', async () => {
  const nextHandler = (registerData: GuestRegisterData) => {
    console.log(`Entered data: ${registerData}`)
  }

  const testData = getEmptyGuestData()
  testData.authentication_method = AuthenticationMethod.Feide
  testData.first_name = 'Test'
  testData.last_name = 'Tester'

  render(
    <LocalizationProvider dateAdapter={AdapterDateFns}>
      <GuestRegisterStep
        nextHandler={nextHandler}
        initialGuestData={testData}
        registerData={null}
      />
    </LocalizationProvider>
  )

  await waitFor(() => {
    expect(screen.queryByTestId('first-name-input-text')).toBeDisabled()
    expect(screen.queryByTestId('last-name-input-text')).toBeDisabled()
  })
})

test('Name editable if missing and invite is Feide', async () => {
  const nextHandler = (registerData: GuestRegisterData) => {
    console.log(`Entered data: ${registerData}`)
  }

  const testData = getEmptyGuestData()
  testData.authentication_method = AuthenticationMethod.Feide

  render(
    <LocalizationProvider dateAdapter={AdapterDateFns}>
      <GuestRegisterStep
        nextHandler={nextHandler}
        initialGuestData={testData}
        registerData={null}
      />
    </LocalizationProvider>
  )

  await waitFor(() => {
    expect(screen.queryByTestId('first-name-input-text')).toBeEnabled()
    expect(screen.queryByTestId('last-name-input-text')).toBeEnabled()
  })
})

test('Identifier fields disabled if invite is Feide and national ID present', async () => {
  const nextHandler = (registerData: GuestRegisterData) => {
    console.log(`Entered data: ${registerData}`)
  }

  const testData = getEmptyGuestData()
  testData.authentication_method = AuthenticationMethod.Feide
  testData.fnr = '12042335418'

  render(
    <LocalizationProvider dateAdapter={AdapterDateFns}>
      <GuestRegisterStep
        nextHandler={nextHandler}
        initialGuestData={testData}
        registerData={null}
      />
    </LocalizationProvider>
  )

  await waitFor(() => {
    expect(screen.queryByTestId('national-id-number-input')).toBeDisabled()
    expect(screen.queryByTestId('passport_number_input')).toBeDisabled()
    expect(screen.queryByTestId('passport-nationality-input')).toBeDisabled()
  })
})

test('Identifier fields enabled if invite is Feide but national ID number missing', async () => {
  const nextHandler = (registerData: GuestRegisterData) => {
    console.log(`Entered data: ${registerData}`)
  }

  const testData = getEmptyGuestData()
  testData.authentication_method = AuthenticationMethod.Feide
  testData.fnr = ''

  render(
    <LocalizationProvider dateAdapter={AdapterDateFns}>
      <GuestRegisterStep
        nextHandler={nextHandler}
        initialGuestData={testData}
        registerData={null}
      />
    </LocalizationProvider>
  )

  await waitFor(() => {
    expect(screen.queryByTestId('national-id-number-input')).toBeEnabled()
    expect(screen.queryByTestId('passport_number_input')).toBeEnabled()
    expect(screen.queryByTestId('passport-nationality-input')).toBeEnabled()
  })
})

test('Identifier fields disabled and name fields enabled if invite is ID-porten', async () => {
  const nextHandler = (registerData: GuestRegisterData) => {
    console.log(`Entered data: ${registerData}`)
  }

  const testData = getEmptyGuestData()
  testData.authentication_method = AuthenticationMethod.Feide
  testData.fnr = ''

  render(
    <LocalizationProvider dateAdapter={AdapterDateFns}>
      <GuestRegisterStep
        nextHandler={nextHandler}
        initialGuestData={testData}
        registerData={null}
      />
    </LocalizationProvider>
  )

  await waitFor(() => {
    expect(screen.queryByTestId('national-id-number-input')).toBeEnabled()
    expect(screen.queryByTestId('passport_number_input')).toBeEnabled()
    expect(screen.queryByTestId('passport-nationality-input')).toBeEnabled()
  })
})

test('Gender required to be set if gender field is showing', async () => {
  const nextHandler = (registerData: GuestRegisterData) => {
    console.log(`Entered data: ${registerData}`)
  }

  const allFeaturesOn = {
    displayContactAtUnit: true,
    displayComment: true,
    displayContactAtUnitGuestInput: true,
    showGenderFieldForGuest: true,
  }

  const formData: GuestRegisterData = {
    firstName: 'Test',
    lastName: 'Test2',
    mobilePhoneCountry: 'NO',
    mobilePhone: '97543980',
    nationalIdNumber: '',
    passportNumber: '12345678',
    passportNationality: 'NO',
    dateOfBirth: addYears(-20)(new Date()),
    gender: null,
  }

  const testData = getEmptyGuestData()

  // Need this to be able to call on the submit-method on the form
  const reference = {
    current: {
      doSubmit: jest.fn(),
    },
  }

  render(
    <FeatureContext.Provider value={allFeaturesOn}>
      <LocalizationProvider dateAdapter={AdapterDateFns}>
        <GuestRegisterStep
          nextHandler={nextHandler}
          initialGuestData={testData}
          registerData={formData}
          ref={reference}
        />
      </LocalizationProvider>
    </FeatureContext.Provider>
  )

  reference.current?.doSubmit()

  // The validation should fail for gender since to value has been set
  const validationMessage = await waitFor(() =>
    screen.getByText('validation.genderIsRequired')
  )
  expect(validationMessage).toBeInTheDocument()
})

test('Gender not required to be set if gender field is not showing', async () => {
  const nextHandler = (registerData: GuestRegisterData) => {
    console.log(`Entered data: ${registerData}`)
  }

  const showGenderFieldOff = {
    displayContactAtUnit: true,
    displayComment: true,
    displayContactAtUnitGuestInput: true,
    showGenderFieldForGuest: false,
  }

  const formData: GuestRegisterData = {
    firstName: 'Test',
    lastName: 'Test2',
    mobilePhoneCountry: 'NO',
    mobilePhone: '97543980',
    nationalIdNumber: '',
    passportNumber: '12345678',
    passportNationality: 'NO',
    dateOfBirth: addYears(-20)(new Date()),
    gender: null,
  }

  const testData = getEmptyGuestData()
  const reference = {
    current: {
      doSubmit: jest.fn(),
    },
  }

  render(
    <FeatureContext.Provider value={showGenderFieldOff}>
      <LocalizationProvider dateAdapter={AdapterDateFns}>
        <GuestRegisterStep
          nextHandler={nextHandler}
          initialGuestData={testData}
          registerData={formData}
          ref={reference}
        />
      </LocalizationProvider>
    </FeatureContext.Provider>
  )

  reference.current?.doSubmit()
  await waitFor(
    () => {
      expect(
        screen.queryByText('validation.genderIsRequired')
      ).not.toBeInTheDocument()
    },
    {
      timeout: 5000,
    }
  )
})
