import { Link, useParams, useHistory } from 'react-router-dom'

import Page from 'components/page'
import RoleLine from 'components/roleLine'
import { useTranslation } from 'react-i18next'
import {
  Button,
  Table,
  TableBody,
  Box,
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
  TextField,
  Typography,
} from '@mui/material'
import {
  TableContainer,
  TableHead,
  TableHeadCell,
  TableRow,
  TableCell,
} from 'components/table'
import { Guest } from 'interfaces'
import SponsorInfoButtons from 'routes/components/sponsorInfoButtons'
import { useEffect, useState } from 'react'
import { SubmitHandler, useForm } from 'react-hook-form'
import IdentityLine from 'components/identityLine'
import { isValidEmail, submitJsonOpts } from 'utils'

type GuestInfoParams = {
  pid: string
}

type GuestInfoProps = {
  guest: Guest
  updateEmail: (validEmail: string) => void
  resend: () => void
  reloadGuests: () => void
  reloadGuest: () => void
}

type CancelConfirmationDialogProps = {
  open: boolean
  onClose: (ok: boolean) => void
}

function CancelConfirmationDialog({
  open,
  onClose,
}: CancelConfirmationDialogProps) {
  const [t] = useTranslation(['common'])

  const handleCancel = () => {
    onClose(false)
  }

  const handleOk = () => {
    onClose(true)
  }

  return (
    <Dialog open={open}>
      <DialogTitle>{t('confirmationDialog.cancelInvitation')}</DialogTitle>

      <DialogContent>
        {t('confirmationDialog.cancelInvitationDescription')}
      </DialogContent>
      <DialogActions>
        <Button autoFocus color="secondary" onClick={handleCancel}>
          {t('button.cancel')}
        </Button>
        <Button autoFocus color="secondary" onClick={handleOk}>
          {t('button.ok')}
        </Button>
      </DialogActions>
    </Dialog>
  )
}

type Email = {
  email: string
}

export default function GuestInfo({
  guest,
  updateEmail,
  resend,
  reloadGuests,
  reloadGuest,
}: GuestInfoProps) {
  const { pid } = useParams<GuestInfoParams>()
  const [t] = useTranslation(['common'])
  const history = useHistory()
  const [confirmationDialogOpen, setConfirmationDialogOpen] = useState(false)

  // Using useForm even though only the e-mail is allow to change at present, since useForm makes setup and validation easier
  const {
    handleSubmit,
    setValue,
    register,
    reset,
    formState: { errors, isValid, isDirty },
  } = useForm<Email>({
    mode: 'onChange',
  })

  const submit: SubmitHandler<Email> = (data) => {
    updateEmail(data.email)
  }
  const onSubmit = handleSubmit(submit)

  const handleCancel = () => {
    setConfirmationDialogOpen(true)
  }

  const cancelInvitation = () => {
    // There is no body for this request, but using submitJsonOpts still to
    // set the CSRF-token
    fetch(
      `/api/ui/v1/invite/?person_id=${guest.pid}`,
      submitJsonOpts('DELETE', {})
    )
      .then((res) => {
        if (!res.ok) {
          return null
        }
        return res.text()
      })
      .then((result) => {
        if (result !== null) {
          // The invite for the guest has been cancelled, send the user back to the sponsor front page
          reloadGuests()
          history.push('/sponsor')
        }
      })
      .catch((error) => {
        // TODO User should get some feedback telling him something failed
        console.log('error', error)
      })
  }

  const handleDialogClose = (ok: boolean) => {
    setConfirmationDialogOpen(false)
    if (ok) {
      cancelInvitation()
    }
  }

  const { ref, onChange, ...inputProps } = register('email', {
    validate: isValidEmail,
  })

  useEffect(() => {
    reset() // Forces defaultValue to be set to the new value, making the form not dirty
    setValue('email', guest.email)
  }, [guest])

  return (
    <Page>
      <SponsorInfoButtons to="/sponsor" name={`${guest.first} ${guest.last}`} />
      <form onSubmit={onSubmit}>
        <Box sx={{ marginBottom: '2rem' }}>
          <Typography variant="h2">{t('guestInfo.contactInfo')}</Typography>
          <Typography variant="body1">
            {t('guestInfo.contactInfoBody')}
          </Typography>

          <TableContainer>
            <Table sx={{ minWidth: 650 }} aria-label="simple table">
              <TableHead>
                <TableRow>
                  <TableHeadCell align="left" colSpan={2}>
                    {t('guestInfo.contactInfoTableText')}

                  </TableHeadCell>
                </TableRow>
              </TableHead>
              <TableBody>
                <TableRow>
                  <TableCell align="left">{t('input.fullName')}</TableCell>
                  <TableCell align="left">
                    {`${guest.first} ${guest.last}`}
                  </TableCell>
                  <TableCell />
                </TableRow>
                <TableRow>
                  <TableCell align="left">{t('input.email')}</TableCell>
                  <TableCell align="left">
                    <Box
                      sx={{
                        display: 'flex',
                        flexDirection: { xs: 'column', md: 'row' },
                        justifyContent: 'flex-start',
                      }}
                    >
                      <TextField
                        InputLabelProps={{ shrink: true }}
                        id="email"
                        label={t('input.email')}
                        error={!!errors.email}
                        helperText={errors?.email?.message}
                        defaultValue={guest.email}
                        inputRef={ref}
                        onChange={onChange}
                          {...inputProps}
                      />

                      {/* If the guest has not completed the registration process, he should have an invitation he has not responded to */}
                      {!guest.registered && (
                        <Box
                          sx={{
                            display: 'flex',
                            flexDirection: 'row',
                            justifyContent: {
                              xs: 'flex-start',
                              md: 'flex-end',
                            },
                          }}
                        >
                          <Button
                            color="secondary"
                            sx={{
                              maxHeight: '2.3rem',
                              marginLeft: { xs: '0rem', md: '1rem' },
                              marginTop: { xs: '0.3rem', md: '0rem' },
                            }}
                            onClick={resend}
                          >
                            {t('button.resendInvitation')}
                          </Button>
                          <Button
                            color="secondary"
                            sx={{
                              maxHeight: '2.3rem',
                              marginLeft: '0.5rem',
                              marginTop: { xs: '0.3rem', md: '0rem' },
                            }}
                            onClick={handleCancel}
                          >
                            {t('button.cancelInvitation')}
                          </Button>
                        </Box>
                      )}
                    </Box>
                    <CancelConfirmationDialog
                      open={confirmationDialogOpen}
                      onClose={handleDialogClose}
                    />
                  </TableCell>
                  <TableCell />
                </TableRow>
                <TableRow>
                  <TableCell align="left">{t('input.mobilePhone')}</TableCell>
                  <TableCell align="left">{guest.mobile}</TableCell>
                </TableRow>
                <TableRow>
                  <TableCell align="left">{t('feideId')}</TableCell>
                  <TableCell align="left">{guest.feide_id}</TableCell>
                </TableRow>

                <IdentityLine
                  text={t('input.nationalIdNumber')}
                  identity={guest.fnr}
                  reloadGuest={reloadGuest}
                  reloadGuests={reloadGuests}
                />
                <IdentityLine
                  text={t('input.passportNumber')}
                  identity={guest.passport}
                  reloadGuest={reloadGuest}
                  reloadGuests={reloadGuests}
                />
              </TableBody>
            </Table>
          </TableContainer>
          <Button
            type="submit"
            color="secondary"
            disabled={!isValid || !isDirty}
          >
            {t('button.save')}
          </Button>
        </Box>
      </form>
      <Typography variant="h2">{t('guestInfo.roleInfoHead')}</Typography>
      <Typography variant="body1">{t('guestInfo.roleInfoBody')}</Typography>
      <TableContainer>
        <Table sx={{ minWidth: 650 }} aria-label="simple table">
          <TableHead>
            <TableRow>
              <TableHeadCell align="left">
                {t('guestInfo.roleInfoTableText')}
              </TableHeadCell>
              <TableHeadCell align="left" />
              <TableHeadCell align="left" />
              <TableHeadCell align="left" />
            </TableRow>
          </TableHead>
          <TableBody>
            <TableRow>
              <TableHeadCell align="left">{t('common:role')}</TableHeadCell>
              <TableHeadCell align="left">{t('common:ou')}</TableHeadCell>
              <TableHeadCell align="left">{t('common:period')}</TableHeadCell>
              <TableHeadCell align="left" />
            </TableRow>
            {guest.roles.map((role) => (
              <RoleLine key={role.id} pid={pid} role={role} />
            ))}
            <TableRow>
              <TableCell align="left">
                <Button
                  variant="contained"
                  color="secondary"
                  component={Link}
                  to={`/sponsor/guest/${pid}/newrole`}
                >
                  {t('sponsor.addRole')}
                </Button>
              </TableCell>
            </TableRow>
          </TableBody>
        </Table>
      </TableContainer>
    </Page>
  )
}
